// VinciaNJETinterface.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2015 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Functions to interface to matrix elements generated with NJET

#ifndef VINCIA_NJETinterface_H
#define VINCIA_NJETinterface_H

// Check if Vincia-NJET2 flag is set (-DVNJET2)
#ifdef VNJET2

// Include NJET headers
//#include "

// Include Pythia and headers
#include "Pythia8/Basics.h"
//#include "Pythia8/Event.h"
//#include "Pythia8/Info.h"
//#include "Pythia8/ParticleData.h"
//#include "Pythia8/PythiaStdlib.h"
//#include "Pythia8/Settings.h"
//#include "Pythia8/StandardModel.h"
//Used for ostream_iterator, which is used to convert a vector of integers
//into a vector of doubles
//#include <iostream>
//#include <iterator>

// Include Vincia Standard Library
#include "VinciaCommon.h"

// Stdlib header for complex numbers.
#include <complex>

using namespace std;
using namespace Pythia8;

namespace Vincia {

  // Amplitude-Squared Base Class for VINCIA
  class MatrixElement {

  public:
    
    // Constructor.
    MatrixElement() : isInitPtr(false), isInit(false) {};

    // Destructor
    virtual ~MatrixElement() {};

    // NLegs
    virtual unsigned int nLegs() {return 4;}

    // Set Momenta (unpolarized)
    virtual bool setMomenta(vector<Vec4>& momIn) {
      if (momIn.size() != nLegs()) return false;
      momSav = momIn;
      for (int i=0; i<(int)momSav.size(); ++i) helSav.push_back(9);
      // Function to call if anything needs to be translated.
      translateMom();
      return true;
    };

    // Set helicities (assuming momenta already specified)
    virtual bool setHelicities(vector<int>& helIn) {      
      if (helIn.size() == momSav.size() && helIn.size() == nLegs()) {
        helSav = helIn;
        // Function to call if anything needs to be translated.
        translateHel();
        return true;
      } else return false;
    }
      
    // Wrapper from Particle Class
    virtual bool setMomHel(vector<Particle> partonsIn) {
      if (partonsIn.size() != nLegs()) return false;
      for (int i=0; i<(int)partonsIn.size(); ++i) {
        momSav.push_back(partonsIn[i].p());
        helSav.push_back(partonsIn[i].pol());
      }
      translateMom();
      translateHel();
      return true;
    }
    
    // Pure virtual functions to encapsulate translation to code-
    // specific momentum or helicity conventions
    virtual void translateMom() {};
    virtual void translateHel() {};
    
    // Return Matrix Element, with unit couplings and no averages
    double getAmp2() { return 0.;};

  protected:

    vector<Vec4> momSav;
    vector<int> helSav;

    // Initialized y/n
    bool isInitPtr, isInit;
    
  };

  // Derived class containing structures specific to NJET2
  
  class MEnjet2 : public MatrixElement {

  public:

    // Constructor.
    MEnjet2() { 
    };
    
    // Destructor
    virtual ~MEnjet2() {};

    // Functions to translate from VINCIA momenta and helicities to NJET ones
    virtual void translateMom() {
      
    }

  protected:

    NJetAmp<double>* sqamp;
    
  };
}
#else
// Put dummy interface definitions here, if needed.
#endif
#endif
